function Arc= WeightOptimization(Global,G2,Population,wD,N)
% The second step of CLEA, which aims to search the PF according to the
% bi-direction weight vectors

%--------------------------------------------------------------------------
% The copyright of the PlatEMO belongs to the BIMK Group. You are free to
% use the PlatEMO for research purposes. All publications which use this
% platform or any code in the platform should acknowledge the use of
% "PlatEMO" and reference "Ye Tian, Ran Cheng, Xingyi Zhang, and Yaochu
% Jin, PlatEMO: A MATLAB Platform for Evolutionary Multi-Objective
% Optimization, 2016".
%--------------------------------------------------------------------------

% Copyright (c) 2016-2017 Cheng He
	%% Choose NF solutions as the reference solutions
	Reference    = max(Population.objs,[],1);
    [RefPop,~,~] = EnvironmentalSelection(Population,wD);
    
    %% Calculate the reference directions
	lower       = Global.lower;
    upper       = Global.upper;
	Direction   = [sum((RefPop.decs-repmat(lower,wD,1)).^2,2).^(0.5);sum((repmat(upper,wD,1)-RefPop.decs).^2,2).^(0.5)];
	Direct      = [(RefPop.decs-repmat(lower,wD,1));(repmat(upper,wD,1)-RefPop.decs)]./repmat(Direction,1,Global.D);
	wmax        = sum((upper-lower).^2)^(0.5)*0.5;
    
    %% Optimize the weight variables by DE
	w0 = rand(N,2*wD).*wmax;                           % Initialize the population
    [fitness,PopNew] = fitfunc(w0,Direct,Global,Reference);    % Calculate the fitness and store the solutions
	Arc = PopNew(NDSort(PopNew.objs,1)==1);
	pCR = 0.2;
    beta_min=0.2;   % Lower Bound of Scaling Factor
    beta_max=0.8;   % Upper Bound of Scaling Factor
    empty_individual.Position=[];
    empty_individual.Cost=[];
    pop=repmat(empty_individual,N,1);
    for i = 1 : N
        pop(i).Position = w0(i,:);
        pop(i).Cost = fitness(i);
    end
    temp = [];
    for it = 1 : G2
        for i = 1 : N
            x = pop(i).Position;
            A = randperm(N);
            A(A==i) = [];
            a = A(1); b = A(2); c = A(3);
            % Mutation  %beta=unifrnd(beta_min,beta_max);
            beta = unifrnd(beta_min,beta_max,[1 2*wD]);
            y = pop(a).Position + beta.*(pop(b).Position - pop(c).Position);
            y = min(max(y,0),wmax);
            % Crossover
            z = zeros(size(x));
            j0=randi([1 numel(x)]);
            for j=1:numel(x)
                if j==j0 || rand<=pCR
                    z(j) = y(j);
                else
                    z(j) = x(j);
                end
            end
            NewSol.Position = z;
            [fit,PopNew] = fitfunc(z,Direct,Global,Reference);
            temp = [temp,PopNew];
            temp = temp(NDSort(temp.objs,1)==1);
            NewSol.Cost = fit;
            if NewSol.Cost < pop(i).Cost
                pop(i)=NewSol;
            end
        end
    end
    %Update and store the non-dominated solutions
    Arc = [Arc,temp];
    if length(Arc) > Global.N
        [frontNo,~] = NDSort(Arc.objs,1);
        Arc = Arc(frontNo==1);
    end
end

function [Obj,OffSpring] = fitfunc(w0,direct,Global,Reference)
    [SubN,WD] = size(w0); 
    WD      = WD/2;
    Obj   	= zeros(SubN,1);
    OffSpring  = [];
    for i = 1 : SubN 
        PopDec  = [repmat(w0(i,1:WD)',1,Global.D).*direct(1:WD,:)+repmat(Global.lower,WD,1);
                      repmat(Global.upper,WD,1) - repmat(w0(i,WD+1:end)',1,Global.D).*direct(WD+1:end,:)];
        OffWPop = INDIVIDUAL(PopDec);
        OffSpring  = [OffSpring,OffWPop];
        Obj(i)  = -HV(OffWPop.objs,Reference);
    end
end
